﻿using EDMXParserV1;
using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Text;

namespace MVCGenerator
{
    public class ControllersCS
    {
        private SpaceStackStringBuilder sb = new SpaceStackStringBuilder();

        String NamespaceName = "";
        List<GridChoice> GridChoices;
        List<Searchable> SearchChoices;
        List<Sortable> SortChoices;
        String EntityPropertyName;
        String PropertyClassName;
        String PropertyClassNameVariable;
        String PropertyClassPrimaryKeyName;
        String PropertyClassPrimaryKeyType; 
        bool UsingDropDownLists;
        EntitySet SelectedEntitySet;
        EDMXParserV1.Edmx SelectedEDMX;
        
        public ControllersCS(
            String InNamespaceName, 
            List<GridChoice> InGridChoices, 
            List<Searchable> InSearchChoices, 
            List<Sortable> InSortChoices, 
            String InEntityPropertyName, 
            String InPropertyClassName, 
            String InPropertyClassNameVariable, 
            String InPropertyClassPrimaryKeyName, 
            String InPropertyClassPrimaryKeyType, 
            bool InUsingDropDownLists, 
            EntitySet InSelectedEntitySet, 
            EDMXParserV1.Edmx InSelectedEDMX
        )
        {
            NamespaceName = InNamespaceName;
            GridChoices = InGridChoices;
            SearchChoices = InSearchChoices;

            SortChoices = InSortChoices;
            EntityPropertyName = InEntityPropertyName; 
            PropertyClassName = InPropertyClassName;
            PropertyClassNameVariable = InPropertyClassNameVariable; 
            PropertyClassPrimaryKeyName = InPropertyClassPrimaryKeyName;
            PropertyClassPrimaryKeyType = InPropertyClassPrimaryKeyType;
            UsingDropDownLists = InUsingDropDownLists; 
            SelectedEntitySet = InSelectedEntitySet;
            SelectedEDMX = InSelectedEDMX;

            sb = new SpaceStackStringBuilder();
            sb.AppendLine("using System;");
            sb.AppendLine("using System.Collections.Generic;");
            sb.AppendLine("using System.Data;");
            sb.AppendLine("using System.Data.Objects;");
            sb.AppendLine("using System.Data.Entity;");
            sb.AppendLine("using System.Linq;");
            sb.AppendLine("using System.Web;");
            sb.AppendLine("using System.Web.Mvc;");
            sb.AppendLine("using " + NamespaceName + ";");
            sb.AppendLine("using " + NamespaceName + ".Models;");
            sb.AppendLine("");
            sb.AppendLine("namespace " + NamespaceName + ".Controllers");
            sb.AppendLine("{");
            sb.AddSpace();
            if (true)
            {
                sb.AppendLine("public class " + EntityPropertyName + "Controller : Controller");
                sb.AppendLine("{");
                //            sb.AppendLine("        private " + EntityClass.Name + " db = new " + EntityClass.Name + "();");
                String EntityClassName = SelectedEDMX.Runtime.Mappings.Mapping.EntityContainerMapping.CdmEntityContainer;
                sb.AddSpace();
                if (true)
                {
                    sb.AppendLine("private " + EntityClassName + " db = new " + EntityClassName + "();");

                    AddGetIndexMethod();


                    AddIndexPostMethod();

                    foreach (Searchable SearchItem in SearchChoices)
                    {
                        AddClassFilterMethod(SearchItem);
                    }

                    AddConfigureSortingMethod();

                    // Paging Section
                    AddConfigurePagingMethod(EntityPropertyName, PropertyClassName);                        
                    AddGetNumberOfPagesToSkipMethod(PropertyClassName);
                    AddGetLastPageMethod(PropertyClassName);
                    //End Paging Section

                    AddGetClassDetailFromEntityMethod();
                    AddGetDateDropDownListOptionsMethod();
                    AddGetNumericalDropDownListOptionsMethod();
                    AddGetBooleanDropDownListOptionsMethod();
                    
                    AddDetailsMethod();

                    AddCreateGetMethod();
                    AddCreatePostMethod();

                    AddEditGetMethod();
                    AddEditPostMethod();

                    AddDeleteGetMethod();
                    AddDeletePostMethod();
                    
                    AddDisposeMethod();
                }
                sb.RemoveSpace();
                sb.AppendLine("}");
            }
            sb.RemoveSpace();
            sb.AppendLine("}");  // end Namespace

        }

        private void AddGetIndexMethod()
        {
            sb.AppendLine();
            sb.AppendLine("//");
            sb.AppendLine("// GET: /" + EntityPropertyName + "/");
            sb.AppendLine();
            sb.AppendLine("public ViewResult Index()");
            sb.AppendLine("{");

            sb.AddSpace();
            if (true)
            {
                GetIndexBaseQuery();

                if (GridChoices.Count > 0)
                {
                    sb.AppendLine();
                    sb.AppendLine("// Get the Query for the Details it doesn't execute it yet");
                    sb.AppendLine("var " + PropertyClassName + "Details = Get" + PropertyClassName + "DetailFrom" + EntityPropertyName + "(" + EntityPropertyName + ");");
                }

                sb.AppendLine();
                sb.AppendLine(PropertyClassName + "Index " + PropertyClassName + "Index = new " + PropertyClassName + "Index();");

                // Paging Section
                sb.AppendLine();
                sb.AppendLine("// Set the number of Records per page that will be displayed.");
                sb.AppendLine(PropertyClassName + "Index.RecordsPerPage = 10;");
                sb.AppendLine();
                sb.AppendLine("// Set the page number to the first page when going to the page for the first time.");
                sb.AppendLine(PropertyClassName + "Index.CurrentPage = 1;");
                sb.AppendLine();
                sb.AppendLine("// Run the Query the maximum number of records available.");
                sb.AppendLine("// This executes a database action");
                sb.AppendLine(PropertyClassName + "Index.RecordCount = " + EntityPropertyName + ".Count();");

                sb.AppendLine();
                sb.AppendLine("// Configures the database sort order Order by settings of the query being built");
                sb.AppendLine(EntityPropertyName + " = ConfigureSorting(" + PropertyClassName + "Index, " + EntityPropertyName + ");");
                sb.AppendLine();
                sb.AppendLine("// Configures the Paging by settings of the query being built");
                sb.AppendLine(EntityPropertyName + " = ConfigurePaging(" + PropertyClassName + "Index, " + EntityPropertyName + ");");
                // Done paging

                sb.AppendLine();
                sb.AppendLine("// Get the Details from the database with the query + sorting + paging");
                sb.AppendLine("// This exectues a database action");
                sb.AppendLine(PropertyClassName + "Index." + PropertyClassName + "Details = Get" + PropertyClassName + "DetailFrom" + EntityPropertyName + "(" + EntityPropertyName + ");");

                foreach (Searchable item in SearchChoices)
                {
                    bool UsingDDL = false;
                    if (item.GridChoice.IsNull())
                    {
                        // do nothing
                    }
                    else
                    {
                        sb.AppendLine();
                        sb.AppendLine("// populate the dropdown for the " + item.ColumnName + " column");
                        sb.AppendStart();
                        sb.Append(PropertyClassName + "Index." + item.ColumnName + "SearchOption = new ");
                        sb.Append(GetSelectListFromGridChoice(SelectedEDMX, item.GridChoice, "", SelectedEntitySet));
                        sb.AppendLine();
                        UsingDDL = true;
                    }
                    if (UsingDDL == false)
                    {
                        if (item.Property.Type.Contains("DateTime"))
                        {
                            sb.AppendLine("");
                            sb.AppendLine("// The type of search to perform using date options");
                            sb.AppendLine(PropertyClassName + "Index." + item.ColumnName + "SearchOption = GetDateDropDownListOptions();");
                        }
                        if (Basics.IsNumericType(item.Property.Type))
                        {
                            sb.AppendLine("");
                            sb.AppendLine("// The type of search to perform using numerical options");
                            sb.AppendLine(PropertyClassName + "Index." + item.ColumnName + "SearchOption = GetNumericalDropDownListOptions();");
                        }
                        else if (item.Property.Type.Contains("Boolean"))
                        {
                            sb.AppendLine("");
                            sb.AppendLine("// The type of search to perform using boolean dropdown options");
                            sb.AppendLine(PropertyClassName + "Index." + item.ColumnName + "SearchOption = GetBooleanDropDownListOptions();");
                        }
                    }
                }
                sb.AppendLine();
                sb.AppendLine("return View(" + PropertyClassName + "Index);");
                sb.RemoveSpace();

                sb.AppendLine("}");
            }
        }

        private void AddDeletePostMethod()
        {
            sb.AppendLine();
            sb.AppendLine("//");
            sb.AppendLine("// POST:  /" + EntityPropertyName + "/Delete/5");
            sb.AppendLine();
            sb.AppendLine(@"[HttpPost, ActionName(""Delete"")]");
            sb.AppendLine("public ActionResult DeleteConfirmed(" + PropertyClassPrimaryKeyType + " id)");
            sb.AppendLine("{");
            if (true)
            {
                sb.AddSpace();
                sb.AppendLine(PropertyClassName + " " + PropertyClassNameVariable + " = db." + EntityPropertyName + ".Single(o => o." + PropertyClassPrimaryKeyName + " == id);");
                sb.AppendLine("db." + EntityPropertyName + ".DeleteObject(" + PropertyClassNameVariable + ");");
                sb.AppendLine("db.SaveChanges();");
                sb.AppendLine(@"return RedirectToAction(""Index"");");
                sb.RemoveSpace();
            }
            sb.AppendLine("}");
        }

        private void AddDeleteGetMethod()
        {
            sb.AppendLine();
            sb.AppendLine("//");
            sb.AppendLine("// GET:  /" + EntityPropertyName + "/Delete/5");
            sb.AppendLine();
            sb.AppendLine("public ActionResult Delete(" + PropertyClassPrimaryKeyType + " id)");
            sb.AppendLine("{");
            if (true)
            {
                sb.AddSpace();
                sb.AppendLine("" + PropertyClassName + " " + PropertyClassNameVariable + " = db." + EntityPropertyName + ".Single(o => o." + PropertyClassPrimaryKeyName + " == id);");
                sb.AppendLine("return View(" + PropertyClassNameVariable + ");");
                sb.RemoveSpace();
            }
            sb.AppendLine("}");
        }

        private void AddEditPostMethod()
        {
            sb.AppendLine();
            sb.AppendLine("//");
            sb.AppendLine("// POST:  /" + EntityPropertyName + "/Edit/5");
            sb.AppendLine();
            sb.AppendLine("[HttpPost]");
            if (UsingDropDownLists)
            {
                sb.AppendLine("public ActionResult Edit(" + PropertyClassName + "CreateOrEdit " + PropertyClassNameVariable + ")");
            }
            else
            {
                sb.AppendLine("public ActionResult Edit(" + PropertyClassName + " " + PropertyClassNameVariable + ")");
            }

            sb.AppendLine("{");

            if (true)
            {
                sb.AddSpace();

                sb.AppendLine("if (ModelState.IsValid)");
                sb.AppendLine("{");
                if (true)
                {
                    sb.AddSpace();
                    if (UsingDropDownLists)
                    {
                        sb.AppendLine(PropertyClassName + " Current" + PropertyClassName + " = new " + PropertyClassName + "();");


                        foreach (var item in SelectedEntitySet.EntityTypeLink.Properties)
                        {
                            bool GridChoicesFound = false;
                            foreach (GridChoice GridChoice in GridChoices)
                            {
                                if (item.Name == GridChoice.ColumnName)
                                {
                                    GridChoicesFound = true;
                                    sb.AppendLine("Current" + PropertyClassName + "." + item.Name + " = " + PropertyClassNameVariable + ".DDL" + item.Name + ";");
                                }
                            }

                            if (GridChoicesFound)
                            {
                                // do nothing
                            }
                            else
                            {
                                sb.AppendLine("Current" + PropertyClassName + "." + item.Name + " = " + PropertyClassNameVariable + "." + item.Name + ";");
                            }
                        }

                        sb.AppendLine();
                        sb.AppendLine("db." + EntityPropertyName + ".Attach( Current" + PropertyClassName + " );");
                        sb.AppendLine();
                        sb.AppendLine("db.ObjectStateManager.ChangeObjectState( Current" + PropertyClassName + ", EntityState.Modified);");
                    }
                    else
                    {
                        sb.AppendLine("db." + EntityPropertyName + ".Attach(" + PropertyClassNameVariable + ");");
                        sb.AppendLine("db.ObjectStateManager.ChangeObjectState(" + PropertyClassNameVariable + ", EntityState.Modified);");
                    }

                    sb.AppendLine("try");
                    sb.AppendLine("{");
                    if (true)
                    {
                        sb.AddSpace();
                        sb.AppendLine("db.SaveChanges();");
                        sb.RemoveSpace();
                    }
                    sb.AppendLine("}");
                    sb.AppendLine("catch ( System.Data.UpdateException Ex )");
                    sb.AppendLine("{");
                    if (true)
                    {
                        sb.AddSpace();
                        sb.AppendLine(@"if (Ex.Message.Contains(""DefiningQuery and no <UpdateFunction>""))");
                        sb.AppendLine("{");
                        if (true)
                        {
                            sb.AddSpace();
                            sb.AppendLine(@"// A primary key is missing on the table in sql server.");
                            sb.AppendLine(@"// Open SQL Server Management studio, find table (tablename) ");
                            sb.AppendLine(@"// Right click on (tablename), click ""Design""");
                            sb.AppendLine(@"// Right click on column (columnName), click ""Set Primary Key""");
                            sb.AppendLine(@"// Save the changes.");
                            sb.AppendLine(@"// You have to update your model, and build the solution ");
                            sb.AppendLine(@"//  ");
                            sb.AppendLine(@"throw new Exception(""A primary key needs to be added to the table in sql server, then update the model, and build the solution"");");
                            sb.RemoveSpace();
                        }
                        sb.AppendLine("}");
                        sb.AppendLine();
                        sb.AppendLine(@"throw ( Ex );");
                        sb.RemoveSpace();
                    }
                    sb.AppendLine("}");
                    sb.AppendLine(@"return RedirectToAction(""Index"");");
                    sb.RemoveSpace();
                }
                sb.AppendLine("}");

                sb.AppendLine("return View(" + PropertyClassNameVariable + ");");
                sb.RemoveSpace();
            }
            sb.AppendLine("}");
        }

        private void AddEditGetMethod()
        {
            sb.AppendLine("");
            sb.AppendLine("//");
            sb.AppendLine("// GET:  /" + EntityPropertyName + "/Edit/5");
            sb.AppendLine();
            sb.AppendLine("public ActionResult Edit(" + PropertyClassPrimaryKeyType + " ID)");
            sb.AppendLine("{");
            if (UsingDropDownLists)
            {
                if (true)
                {
                    sb.AddSpace();
                    sb.AppendLine(PropertyClassName + " SingleItem = db." + EntityPropertyName + ".Single(o => o." + PropertyClassPrimaryKeyName + " == ID);");
                    sb.AppendLine();
                    sb.AppendLine(PropertyClassName + "CreateOrEdit " + PropertyClassName + "CreateOrEdit = new " + PropertyClassName + "CreateOrEdit();");
                    sb.AppendLine();

                    foreach (var item in SelectedEntitySet.EntityTypeLink.Properties)
                    {
                        bool GridChoicesFound = false;
                        foreach (GridChoice GridChoice in GridChoices)
                        {
                            if (item.Name == GridChoice.ColumnName)
                            {
                                GridChoicesFound = true;

                                if (GridChoice.CanLinkedTableBeNull)
                                {
                                    sb.AppendLine();
                                    sb.AppendLine("//The linked column can be null, if the linked column is null show null, otherwise show the value");

                                    sb.AppendStart();
                                    //Example
                                    //ReportCreateOrEdit.DDLReportGroupID = (SingleItem.ReportGroupID == null ? new Nullable<int>() : SingleItem.ReportGroupID.Value);

                                    //ReportCreateOrEdit.DDLReportGroupID = 
                                    sb.Append(PropertyClassName + @"CreateOrEdit.DDL" + GridChoice.ColumnName + " = ");

                                    //(SingleItem.ReportGroupID == null ? new Nullable<int>()
                                    sb.Append("(SingleItem." + GridChoice.ColumnName + " == null ? new Nullable<" + Basics.GetDataTypeFromTableNameColumnName(GridChoice.PrimaryTable, GridChoice.PrimaryKey, SelectedEntitySet) + ">()");

                                    // : SingleItem.ReportGroupID
                                    sb.Append(" : SingleItem." + GridChoice.ColumnName);
                                }
                                else
                                {

                                    sb.AppendStart();
                                    sb.Append(PropertyClassName + @"CreateOrEdit.DDL" + GridChoice.ColumnName + " = SingleItem." + GridChoice.ColumnName);
                                }

                                //if (item.IsNullable() && item.Type != "String")
                                //{
                                //    sb.Append(".Value");
                                //}
                                if (item.IsNullable())
                                {
                                    if (item.Type == "String")
                                    {
                                        // do nothing
                                    }
                                    else
                                    {
                                        sb.Append(".Value");
                                    }
                                }
                                else
                                {
                                    // do nothing
                                }

                                if (GridChoice.CanLinkedTableBeNull)
                                {
                                    // end of the ? statement for null check
                                    sb.Append(")");
                                }

                                sb.Append(";");
                                sb.AppendLine();

                                sb.AppendLine();
                                sb.AppendLine("// Create a select list with key column " + GridChoice.PrimaryKey + " and display column " + GridChoice.DisplayColumnName);
                                sb.AppendStart();
                                sb.Append(PropertyClassName + @"CreateOrEdit." + GridChoice.ColumnName + " = new ");
                                //new SelectList( db., "id", "description" , SingleItem.applicationId);

                                sb.Append(
                                    GetSelectListFromGridChoice(SelectedEDMX, GridChoice, "SingleItem." + GridChoice.ColumnName, SelectedEntitySet)
                                );

                                //                            sb.Append(item.GetSelectList(Lst, NamespaceName, EntityMappingList, "SingleItem"));
                                sb.AppendLine();
                            }
                        }

                        if (GridChoicesFound)
                        {
                            // do nothing
                        }
                        else
                        {
                            sb.AppendLine(PropertyClassName + @"CreateOrEdit." + item.Name + " = SingleItem." + item.Name + ";");
                        }
                    }

                    sb.AppendLine();
                    sb.AppendLine("return View(" + PropertyClassName + @"CreateOrEdit);");


                }
                sb.RemoveSpace();
            }
            else
            {
                sb.AppendLine(PropertyClassName + " " + PropertyClassNameVariable + " = db." + EntityPropertyName + ".Single(o => o." + PropertyClassPrimaryKeyName + " == ID);");
                sb.AppendLine("return View(" + PropertyClassNameVariable + ");");
            }
            sb.AppendLine("}");
        }

        private void AddConfigureSortingMethod()
        {
            Sortable FirstSortableChoice = null;
            if (true)
            {
                sb.AppendLine();
                sb.AppendLine("private static IQueryable<" + PropertyClassName + "> ConfigureSorting(" + PropertyClassName + "Index " + PropertyClassName + "Index, IQueryable<" + PropertyClassName + "> " + EntityPropertyName + ")");
                sb.AppendLine("{");

                if (true)
                {
                    sb.AddSpace();
                    sb.AppendLine("switch (" + PropertyClassName + "Index.SortColumn )");
                    sb.AppendLine("{");
                    if (true)
                    {
                        sb.AddSpace();
                        foreach (Sortable Sortable in SortChoices)
                        {
                            if (FirstSortableChoice.IsNull())
                            {
                                FirstSortableChoice = Sortable;
                            }
                            sb.AppendLine(@"case """ + Sortable.ColumnName + @""":");
                            if (true)
                            {
                                sb.AddSpace();
                                AddSortCode(GridChoices, EntityPropertyName, PropertyClassName, sb, Sortable);
                                sb.AppendLine("break;");
                                sb.RemoveSpace();
                            }

                        }
                        sb.AppendLine("default:");
                        if (true)
                        {
                            sb.AddSpace();
                            AddSortCode(GridChoices, EntityPropertyName, PropertyClassName, sb, FirstSortableChoice);

                            sb.AppendLine("break;");
                            sb.RemoveSpace();
                        }
                        sb.RemoveSpace();
                    }
                    sb.AppendLine("}");
                    sb.AppendLine("return " + EntityPropertyName + ";");
                    sb.RemoveSpace();
                }
                sb.AppendLine("}");
            }
        }

        private void AddClassFilterMethod(Searchable SearchItem)
        {
            //                sb.AppendLine(@"            " + EntityPropertyName + " = Add" + SearchItem.ColumnName + "Filter(" + PropertyClassName + "Index, " + EntityPropertyName + ");");

            sb.AppendLine();
            sb.AppendLine(@"//" + SearchItem.ColumnName);
            sb.AppendLine(@"private static IQueryable<" + PropertyClassName + "> Add" + SearchItem.ColumnName + "Filter(" + PropertyClassName + "Index " + PropertyClassName + "Index, IQueryable<" + PropertyClassName + "> " + EntityPropertyName + ")");
            sb.AppendLine(@"{");
            sb.AddSpace();
            //                sb.AppendLine(@"            " + EntityPropertyName + " = Add" + SearchItem.ColumnName + "Filter(" + PropertyClassName + "Index, " + EntityPropertyName + ");");

            //                    Products = AddProductIDFilter(ProductIndex, Products);


            sb.AppendLine();
            sb.AppendLine("if (" + PropertyClassName + "Index." + SearchItem.ColumnName + "Search != null)");
            sb.AppendLine("{");
            sb.AddSpace();
            if (SearchItem.GridChoice.IsNull())
            {
                if (SearchItem.Property.Type.Contains("DateTime"))
                {
                    sb.AppendLine("switch (" + PropertyClassName + "Index." + SearchItem.ColumnName + "SearchOptionResult )");
                    sb.AppendLine("{");
                    sb.AddSpace();

                    sb.AppendLine(@"case null:");

                    sb.AddSpace();
                    sb.AppendLine(EntityPropertyName + " = " + EntityPropertyName + ".Where(x =>  EntityFunctions.TruncateTime(x." + SearchItem.ColumnName + ") == " + PropertyClassName + "Index." + SearchItem.ColumnName + "Search);");
                    sb.AppendLine("break;");
                    sb.RemoveSpace();

                    sb.AppendLine(@"case ""Equals"":");

                    sb.AddSpace();
                    sb.AppendLine(EntityPropertyName + " = " + EntityPropertyName + ".Where(x =>  EntityFunctions.TruncateTime(x." + SearchItem.ColumnName + ") == " + PropertyClassName + "Index." + SearchItem.ColumnName + "Search);");
                    sb.AppendLine("break;");
                    sb.RemoveSpace();

                    sb.AppendLine(@"case ""Before"":");
                    sb.AddSpace();
                    sb.AppendLine(EntityPropertyName + " = " + EntityPropertyName + ".Where(x => EntityFunctions.TruncateTime(x." + SearchItem.ColumnName + ") < " + PropertyClassName + "Index." + SearchItem.ColumnName + "Search);");
                    sb.AppendLine("break;");
                    sb.RemoveSpace();

                    sb.AppendLine(@"case ""After"":");
                    sb.AddSpace();
                    sb.AppendLine(EntityPropertyName + " = " + EntityPropertyName + ".Where(x => EntityFunctions.TruncateTime(x." + SearchItem.ColumnName + ") > " + PropertyClassName + "Index." + SearchItem.ColumnName + "Search);");
                    sb.AppendLine("break;");
                    sb.RemoveSpace();

                    sb.AppendLine(@"case ""Between"":");
                    sb.AddSpace();
                    sb.AppendLine("if (" + PropertyClassName + "Index." + SearchItem.ColumnName + "SearchBetween != null)");
                    sb.AppendLine("{");
                    sb.AddSpace();
                    sb.AppendLine(EntityPropertyName + " = " + EntityPropertyName + ".Where(x => EntityFunctions.TruncateTime(x." + SearchItem.ColumnName + ") >= " + PropertyClassName + "Index." + SearchItem.ColumnName + "Search);");
                    sb.AppendLine(EntityPropertyName + " = " + EntityPropertyName + ".Where(x => EntityFunctions.TruncateTime(x." + SearchItem.ColumnName + ") <= " + PropertyClassName + "Index." + SearchItem.ColumnName + "SearchBetween);");
                    sb.RemoveSpace();
                    sb.AppendLine("}");
                    sb.AppendLine("else");
                    sb.AppendLine("{");
                    if (true)
                    {
                        sb.AddSpace();
                        sb.AppendLine(EntityPropertyName + " = " + EntityPropertyName + ".Where(x => EntityFunctions.TruncateTime(x." + SearchItem.ColumnName + ") >= " + PropertyClassName + "Index." + SearchItem.ColumnName + "Search);");
                        sb.AppendLine(EntityPropertyName + " = " + EntityPropertyName + ".Where(x => x." + SearchItem.ColumnName + " == null);");
                        sb.RemoveSpace();
                    }
                    sb.AppendLine("}");
                    sb.AppendLine("break;");
                    sb.RemoveSpace();

                    sb.AppendLine("default:");
                    if (true)
                    {
                        sb.AddSpace();
                        sb.AppendLine("break;");
                        sb.RemoveSpace();
                    }
                    sb.RemoveSpace();
                    sb.AppendLine("}");
                }
                else if (Basics.IsNumericType(SearchItem.Property.Type))
                {
                    sb.AppendLine("switch (" + PropertyClassName + "Index." + SearchItem.ColumnName + "SearchOptionResult )");
                    sb.AppendLine("{");
                    if (true)
                    {
                        sb.AddSpace();
                        sb.AppendLine(@"case null:");

                        if (true)
                        {
                            sb.AddSpace();
                            sb.AppendLine(EntityPropertyName + " = " + EntityPropertyName + ".Where(x => x." + SearchItem.ColumnName + " == " + PropertyClassName + "Index." + SearchItem.ColumnName + "Search);");
                            sb.AppendLine("break;");
                            sb.RemoveSpace();
                        }

                        sb.AppendLine(@"case ""="":");

                        if (true)
                        {
                            sb.AddSpace();
                            sb.AppendLine(EntityPropertyName + " = " + EntityPropertyName + ".Where(x => x." + SearchItem.ColumnName + " == " + PropertyClassName + "Index." + SearchItem.ColumnName + "Search);");
                            sb.AppendLine("break;");
                            sb.RemoveSpace();
                        }

                        sb.AppendLine(@"case ""!="":");
                        if (true)
                        {
                            sb.AddSpace();
                            sb.AppendLine(EntityPropertyName + " = " + EntityPropertyName + ".Where(x => x." + SearchItem.ColumnName + " != " + PropertyClassName + "Index." + SearchItem.ColumnName + "Search);");
                            sb.AppendLine("break;");
                            sb.RemoveSpace();
                        }

                        sb.AppendLine(@"case "">"":");
                        if (true)
                        {
                            sb.AddSpace();
                            sb.AppendLine(EntityPropertyName + " = " + EntityPropertyName + ".Where(x => x." + SearchItem.ColumnName + " > " + PropertyClassName + "Index." + SearchItem.ColumnName + "Search);");
                            sb.AppendLine("break;");
                            sb.RemoveSpace();
                        }

                        sb.AppendLine(@"case "">="":");
                        if (true)
                        {
                            sb.AddSpace();
                            sb.AppendLine(EntityPropertyName + " = " + EntityPropertyName + ".Where(x => x." + SearchItem.ColumnName + " >= " + PropertyClassName + "Index." + SearchItem.ColumnName + "Search);");
                            sb.AppendLine("break;");
                            sb.RemoveSpace();
                        }
                        sb.AppendLine(@"case ""<"":");

                        if (true)
                        {
                            sb.AddSpace();
                            sb.AppendLine(EntityPropertyName + " = " + EntityPropertyName + ".Where(x => x." + SearchItem.ColumnName + " < " + PropertyClassName + "Index." + SearchItem.ColumnName + "Search);");
                            sb.AppendLine("break;");
                            sb.RemoveSpace();
                        }
                        sb.AppendLine(@"case ""<="":");

                        if (true)
                        {
                            sb.AddSpace();
                            sb.AppendLine(EntityPropertyName + " = " + EntityPropertyName + ".Where(x => x." + SearchItem.ColumnName + " <= " + PropertyClassName + "Index." + SearchItem.ColumnName + "Search);");
                            sb.AppendLine("break;");
                            sb.RemoveSpace();
                        }

                        sb.AppendLine(@"case ""Between Inclusive"":");
                        if (true)
                        {
                            sb.AddSpace();
                            sb.AppendLine("if (" + PropertyClassName + "Index." + SearchItem.ColumnName + "SearchBetween != null)");
                            sb.AppendLine("{");
                            if (true)
                            {
                                sb.AddSpace();
                                sb.AppendLine(EntityPropertyName + " = " + EntityPropertyName + ".Where(x => x." + SearchItem.ColumnName + " >= " + PropertyClassName + "Index." + SearchItem.ColumnName + "Search);");
                                sb.AppendLine(EntityPropertyName + " = " + EntityPropertyName + ".Where(x => x." + SearchItem.ColumnName + " <= " + PropertyClassName + "Index." + SearchItem.ColumnName + "SearchBetween);");
                                sb.RemoveSpace();
                            }
                            sb.AppendLine("}");
                            sb.AppendLine("else");
                            sb.AppendLine("{");
                            if (true)
                            {
                                sb.AddSpace();
                                sb.AppendLine(EntityPropertyName + " = " + EntityPropertyName + ".Where(x => x." + SearchItem.ColumnName + " >= " + PropertyClassName + "Index." + SearchItem.ColumnName + "Search);");
                                sb.AppendLine(EntityPropertyName + " = " + EntityPropertyName + ".Where(x => x." + SearchItem.ColumnName + " == null);");
                                sb.RemoveSpace();
                            }
                            sb.AppendLine("}");
                            sb.AppendLine("break;");
                            sb.RemoveSpace();
                        }
                        sb.AppendLine(@"case ""Not Between Inclusive"":");
                        if (true)
                        {
                            sb.AddSpace();

                            sb.AppendLine("if (" + PropertyClassName + "Index." + SearchItem.ColumnName + "SearchBetween != null)");
                            sb.AppendLine("{");

                            if (true)
                            {
                                sb.AddSpace();
                                sb.AppendLine(EntityPropertyName + " = " + EntityPropertyName + ".Where(x => ! (x." + SearchItem.ColumnName + " >= " + PropertyClassName + "Index." + SearchItem.ColumnName + "Search && x." + SearchItem.ColumnName + " <= " + PropertyClassName + "Index." + SearchItem.ColumnName + "SearchBetween ) );");
                                sb.RemoveSpace();
                            }

                            sb.AppendLine("}");
                            sb.AppendLine("else");
                            sb.AppendLine("{");

                            if (true)
                            {
                                sb.AddSpace();
                                sb.AppendLine(EntityPropertyName + " = " + EntityPropertyName + ".Where(x => x." + SearchItem.ColumnName + " >= " + PropertyClassName + "Index." + SearchItem.ColumnName + "Search);");
                                sb.AppendLine(EntityPropertyName + " = " + EntityPropertyName + ".Where(x => x." + SearchItem.ColumnName + " == null);");
                                sb.RemoveSpace();
                            }

                            sb.AppendLine("}");
                            sb.AppendLine("break;");
                            sb.RemoveSpace();
                        }

                        sb.AppendLine(@"case ""Between Exclusive"":");
                        if (true)
                        {
                            sb.AddSpace();
                            sb.AppendLine("if (" + PropertyClassName + "Index." + SearchItem.ColumnName + "SearchBetween != null)");
                            sb.AppendLine("{");
                            if (true)
                            {
                                sb.AddSpace();
                                sb.AppendLine(EntityPropertyName + " = " + EntityPropertyName + ".Where(x => x." + SearchItem.ColumnName + " > " + PropertyClassName + "Index." + SearchItem.ColumnName + "Search);");
                                sb.AppendLine(EntityPropertyName + " = " + EntityPropertyName + ".Where(x => x." + SearchItem.ColumnName + " < " + PropertyClassName + "Index." + SearchItem.ColumnName + "SearchBetween);");
                                sb.RemoveSpace();
                            }
                            sb.AppendLine("}");
                            sb.AppendLine("else");
                            sb.AppendLine("{");
                            if (true)
                            {
                                sb.AddSpace();

                                sb.AppendLine(EntityPropertyName + " = " + EntityPropertyName + ".Where(x => x." + SearchItem.ColumnName + " > " + PropertyClassName + "Index." + SearchItem.ColumnName + "Search);");
                                sb.AppendLine(EntityPropertyName + " = " + EntityPropertyName + ".Where(x => x." + SearchItem.ColumnName + " == null);");
                                sb.RemoveSpace();
                            }

                            sb.AppendLine("}");
                            sb.AppendLine("break;");
                            sb.RemoveSpace();
                        }
                        sb.AppendLine(@"case ""Not Between Exclusive"":");
                        if (true)
                        {
                            sb.AddSpace();
                            sb.AppendLine("if (" + PropertyClassName + "Index." + SearchItem.ColumnName + "SearchBetween != null)");
                            sb.AppendLine("{");
                            if (true)
                            {
                                sb.AddSpace();
                                sb.AppendLine(EntityPropertyName + " = " + EntityPropertyName + ".Where(x => !(x." + SearchItem.ColumnName + " > " + PropertyClassName + "Index." + SearchItem.ColumnName + "Search && x." + SearchItem.ColumnName + " < " + PropertyClassName + "Index." + SearchItem.ColumnName + "SearchBetween));");
                                sb.RemoveSpace();
                            }
                            sb.AppendLine("}");
                            sb.AppendLine("else");
                            sb.AppendLine("{");
                            if (true)
                            {
                                sb.AddSpace();
                                sb.AppendLine(EntityPropertyName + " = " + EntityPropertyName + ".Where(x => x." + SearchItem.ColumnName + " > " + PropertyClassName + "Index." + SearchItem.ColumnName + "Search);");
                                sb.AppendLine(EntityPropertyName + " = " + EntityPropertyName + ".Where(x => x." + SearchItem.ColumnName + " == null);");
                                sb.RemoveSpace();
                            }
                            sb.AppendLine("}");
                            sb.AppendLine("break;");
                            sb.RemoveSpace();
                        }
                        sb.AppendLine("default:");
                        if (true)
                        {
                            sb.AddSpace();
                            sb.AppendLine("break;");
                            sb.RemoveSpace();
                        }
                        sb.RemoveSpace();
                    }
                    sb.AppendLine("}");
                }
                else if (SearchItem.Property.Type.Contains("Boolean"))
                {
                    sb.AppendLine("switch (" + PropertyClassName + "Index." + SearchItem.ColumnName + "Search )");
                    sb.AppendLine("{");
                    if (true)
                    {
                        sb.AddSpace();
                        sb.AppendLine(@"case null:");
                        if (true)
                        {
                            sb.AddSpace();
                            sb.AppendLine("// do nothing ");
                            sb.AppendLine("break;");
                            sb.RemoveSpace();
                        }
                        sb.AppendLine(@"case ""Yes"":");
                        if (true)
                        {
                            sb.AddSpace();
                            sb.AppendLine(EntityPropertyName + " = " + EntityPropertyName + ".Where(x =>  x." + SearchItem.ColumnName + " == true);");
                            sb.AppendLine("break;");
                            sb.RemoveSpace();
                        }
                        sb.AppendLine(@"case ""No"":");
                        if (true)
                        {
                            sb.AddSpace();
                            sb.AppendLine(EntityPropertyName + " = " + EntityPropertyName + ".Where(x =>  x." + SearchItem.ColumnName + " == false);");
                            sb.AppendLine("break;");
                            sb.RemoveSpace();
                        }
                        sb.AppendLine("default:");
                        if (true)
                        {
                            sb.AddSpace();
                            sb.AppendLine("break;");
                            sb.RemoveSpace();
                        }
                        sb.RemoveSpace();
                    }
                    sb.AppendLine("}");

                }
                else if (SearchItem.Property.Type == "Int32")
                {
                    sb.AppendLine("int " + SearchItem.ColumnName + "Search = Convert.ToInt32(" + PropertyClassName + "Index." + SearchItem.ColumnName + "Search);");
                    sb.AppendLine(EntityPropertyName + " = " + EntityPropertyName + ".Where(x => x." + SearchItem.ColumnName + " == " + SearchItem.ColumnName + "Search);");
                }
                else if (SearchItem.Property.Type == "Double")
                {
                    sb.AppendLine("Double " + SearchItem.ColumnName + "Search = Convert.ToDouble(" + PropertyClassName + "Index." + SearchItem.ColumnName + "Search);");
                    sb.AppendLine(EntityPropertyName + " = " + EntityPropertyName + ".Where(x => x." + SearchItem.ColumnName + " == " + SearchItem.ColumnName + "Search);");
                }
                else if (SearchItem.Property.Type == "Single")
                {
                    sb.AppendLine("Single " + SearchItem.ColumnName + "Search = Convert.ToSingle(" + PropertyClassName + "Index." + SearchItem.ColumnName + "Search);");
                    sb.AppendLine(EntityPropertyName + " = " + EntityPropertyName + ".Where(x => x." + SearchItem.ColumnName + " == " + SearchItem.ColumnName + "Search);");
                }
                else if (SearchItem.Property.Type == "Int16")
                {
                    sb.AppendLine("Int16 " + SearchItem.ColumnName + "Search = Convert.ToInt16(" + PropertyClassName + "Index." + SearchItem.ColumnName + "Search);");
                    sb.AppendLine(EntityPropertyName + " = " + EntityPropertyName + ".Where(x => x." + SearchItem.ColumnName + " == " + SearchItem.ColumnName + "Search);");
                }
                else if (SearchItem.Property.Type == "short?")
                {
                    sb.AppendLine("short? " + SearchItem.ColumnName + "Search = Convert.ToInt16(" + PropertyClassName + "Index." + SearchItem.ColumnName + "Search);");
                    sb.AppendLine(EntityPropertyName + " = " + EntityPropertyName + ".Where(x => x." + SearchItem.ColumnName + " == " + SearchItem.ColumnName + "Search);");
                }
                else
                {
                    sb.AppendLine(EntityPropertyName + " = " + EntityPropertyName + ".Where(x => x." + SearchItem.ColumnName + ".Contains(" + PropertyClassName + "Index." + SearchItem.ColumnName + "Search));");
                }
            }
            else
            {
                String ConverterTypeName = GetConverterFromTypeName(SearchItem.Property.Type);
                if (ConverterTypeName.Length > 0)
                {
                    sb.AppendLine(SearchItem.Property.Type + " " + SearchItem.GridChoice.ColumnName + "Search = Convert." + GetConverterFromTypeName(SearchItem.Property.Type) + "( " + PropertyClassName + "Index." + SearchItem.GridChoice.ColumnName + "Search );");
                }
                else
                {
                    sb.AppendLine(SearchItem.Property.Type + " " + SearchItem.GridChoice.ColumnName + "Search = " + PropertyClassName + "Index." + SearchItem.GridChoice.ColumnName + "Search;");
                }
                sb.AppendLine(EntityPropertyName + " = " + EntityPropertyName + ".Where(x => x." + SearchItem.GridChoice.ColumnName + " == " + SearchItem.GridChoice.ColumnName + "Search);");
            }

            sb.RemoveSpace();
            sb.AppendLine("}");
            sb.AppendLine("return " + EntityPropertyName + ";");
            sb.RemoveSpace();
            sb.AppendLine("}");
        }

        private void AddIndexPostMethod()
        {
            sb.AppendLine();
            sb.AppendLine("[HttpPost]");
            sb.AppendLine("public ViewResult Index(" + PropertyClassName + "Index " + PropertyClassName + "Index)");
            sb.AppendLine("{");
            sb.AddSpace();
            GetIndexBaseQuery();
            //sb.AppendLine("            var Groups = db");
            //sb.AppendLine("            .Groups           ");
            //sb.AppendLine("            .Include( x => x.Application );");
            //sb.AppendLine("");

            foreach (Searchable SearchItem in SearchChoices)
            {
                sb.AppendLine();
                sb.AppendLine(EntityPropertyName + " = Add" + SearchItem.ColumnName + "Filter(" + PropertyClassName + "Index, " + EntityPropertyName + ");");
            }

            if (SortChoices.Count() > 0)
            {
                sb.AppendLine();
                sb.AppendLine(EntityPropertyName + " = ConfigureSorting(" + PropertyClassName + "Index, " + EntityPropertyName + ");");
            }

            sb.AppendLine();
            sb.AppendLine(EntityPropertyName + " = ConfigurePaging(" + PropertyClassName + "Index, " + EntityPropertyName + ");");

            foreach (Searchable item in SearchChoices)
            {
                if (item.GridChoice.IsNull())
                {
                    if (item.Property.Type.Contains("DateTime"))
                    {
                        sb.AppendLine();
                        sb.AppendLine(PropertyClassName + "Index." + item.ColumnName + "SearchOption = GetDateDropDownListOptions();");
                    }
                    else if (Basics.IsNumericType(item.Property.Type))
                    {
                        sb.AppendLine();
                        sb.AppendLine(PropertyClassName + "Index." + item.ColumnName + "SearchOption = GetNumericalDropDownListOptions();");
                    }
                    else if (item.Property.Type.Contains("Boolean"))
                    {
                        sb.AppendLine();
                        sb.AppendLine(PropertyClassName + "Index." + item.ColumnName + "SearchOption = GetBooleanDropDownListOptions();");
                    }
                }
                else
                {
                    //sb.AppendLine("//Searchable + IsNull + T1000");
                    sb.AppendStart();
                    sb.Append(PropertyClassName + "Index." + item.ColumnName + "SearchOption = new ");
                    sb.Append(GetSelectListFromGridChoice(SelectedEDMX, item.GridChoice, PropertyClassName + "Index." + item.ColumnName + "Search", SelectedEntitySet));
                    //sb.Append(item.GridChoice.GetSelectList(Lst, NamespaceName, EntityMappingList));
                    sb.AppendLine();
                }
            }

            sb.AppendLine();
            sb.AppendLine(@"ModelState.Remove(""" + PropertyClassName + @"Details"");");
            //                    sb.AppendLine("            " + PropertyClassName + "Index." + PropertyClassName + "Details =  " + EntityPropertyName + ";");
            //if (GridChoices.Count == 0)
            //{
            //    sb.AppendLine("");
            //    sb.AppendLine("            " + PropertyClassName + "Index." + PropertyClassName + "Details =  " + EntityPropertyName + ";");
            //}
            //else
            //{
            sb.AppendLine();
            sb.AppendLine("// this does a database operation.");
            sb.AppendLine(PropertyClassName + "Index." + PropertyClassName + "Details =  Get" + PropertyClassName + "DetailFrom" + EntityPropertyName + "(" + EntityPropertyName + ");");
            //}

            sb.AppendLine();
            sb.AppendLine("return View(" + PropertyClassName + "Index);");
            sb.RemoveSpace();
            sb.AppendLine("}");
        }

        private void AddCreatePostMethod()
        {
            sb.AppendLine();
            sb.AppendLine("//");
            sb.AppendLine("// POST:  /" + EntityPropertyName + "/Create");
            sb.AppendLine();
            sb.AppendLine("[HttpPost]");

            if (UsingDropDownLists)
            {
                sb.AppendLine("public ActionResult Create(" + PropertyClassName + "CreateOrEdit " + PropertyClassNameVariable + "CreateOrEdit)");
            }
            else
            {
                sb.AppendLine("public ActionResult Create(" + PropertyClassName + " " + PropertyClassNameVariable + ")");
            }
            sb.AppendLine("{");
            if (true)
            {
                sb.AddSpace();
                sb.AppendLine("if (ModelState.IsValid)");
                sb.AppendLine("{");

                if (true)
                {
                    sb.AddSpace();
                    if (UsingDropDownLists)
                    {
                        sb.AppendLine(PropertyClassName + " " + PropertyClassNameVariable + " = new " + PropertyClassName + "();");

                        foreach (var item in SelectedEntitySet.EntityTypeLink.Properties)
                        {
                            bool GridChoicesFound = false;
                            foreach (GridChoice GridChoice in GridChoices)
                            {
                                if (item.Name == GridChoice.ColumnName)
                                {
                                    GridChoicesFound = true;
                                    sb.AppendLine(PropertyClassNameVariable + "." + GridChoice.ColumnName + " = " + PropertyClassNameVariable + "CreateOrEdit.DDL" + GridChoice.ColumnName + ";");
                                }
                            }

                            if (GridChoicesFound)
                            {
                                // do nothing
                            }
                            else
                            {
                                sb.AppendLine(PropertyClassNameVariable + "." + item.Name + " = " + PropertyClassNameVariable + "CreateOrEdit." + item.Name + ";");
                            }
                        }
                    }
                    else
                    {
                        // not UsingLists
                    }
                    sb.AppendLine();
                    sb.AppendLine("db." + EntityPropertyName + ".AddObject(" + PropertyClassNameVariable + ");");
                    sb.AppendLine();
                    sb.AppendLine("try");
                    sb.AppendLine("{");
                    if (true)
                    {
                        sb.AddSpace();
                        sb.AppendLine("db.SaveChanges();");
                        sb.RemoveSpace();
                    }
                    sb.AppendLine("}");
                    sb.AppendLine("catch (System.Data.UpdateException Ex)");
                    sb.AppendLine("{");
                    if (true)
                    {
                        sb.AddSpace();
                        sb.AppendLine(@"if (Ex.Message.Contains(""DefiningQuery and no <InsertFunction>""))");
                        sb.AppendLine("{");
                        if (true)
                        {
                            sb.AddSpace();
                            sb.AppendLine("// gota put a primary key on the table in sql server.");
                            sb.AppendLine("// Open SQL Server Management studio, find table (tablename) ");
                            sb.AppendLine(@"// Right click on (tablename), click ""Design""");
                            sb.AppendLine(@"// Right click on column (columnName), click ""Set Primary Key""");
                            sb.AppendLine("// Save the changes.");
                            sb.AppendLine("// You have to update your model, and build the solution ");
                            sb.AppendLine();
                            sb.AppendLine(@"throw new Exception(""A Primary key is missing on the table in sql server, then update the model, and build the solution"");");
                            sb.RemoveSpace();
                        }
                        sb.AppendLine("}");
                        sb.AppendLine();
                        sb.AppendLine(@"throw ( Ex );");

                        sb.RemoveSpace();
                    }
                    sb.AppendLine("}");
                    sb.AppendLine("");
                    sb.AppendLine(@"return RedirectToAction(""Index"");  ");
                    sb.RemoveSpace();
                }
                sb.AppendLine("}");
                sb.AppendLine();

                if (UsingDropDownLists)
                {
                    sb.AppendLine("return View(" + PropertyClassNameVariable + "CreateOrEdit);");
                }
                else
                {
                    sb.AppendLine("return View(" + PropertyClassNameVariable + ");");
                }
                sb.RemoveSpace();
            }
            sb.AppendLine("}");
        }

        private void AddCreateGetMethod()
        {
            sb.AppendLine();
            sb.AppendLine("//");
            sb.AppendLine("// GET: /" + EntityPropertyName + "/Create");
            sb.AppendLine();
            sb.AppendLine("public ActionResult Create()");
            sb.AppendLine("{");
            if (true)
            {
                sb.AddSpace();

                if (UsingDropDownLists)
                {
                    //GroupCreateOrEdit GroupEdit = new GroupCreateOrEdit();
                    sb.AppendLine(@"" + PropertyClassName + "CreateOrEdit " + PropertyClassName + "Edit = new " + PropertyClassName + "CreateOrEdit();");

                    foreach (var item in GridChoices)
                    {
                        
                            sb.AppendLine();
                            sb.AppendStart();
                            sb.Append(PropertyClassName + "Edit." + item.ColumnName + " = new ");
                            sb.Append(GetSelectListFromGridChoice(SelectedEDMX, item, "", SelectedEntitySet));
                            sb.AppendLine();
                    }

                    sb.AppendLine();
                    sb.AppendLine(@"return View( " + PropertyClassName + "Edit );");
                }
                else
                {
                    sb.AppendLine("return View();");
                }

                sb.RemoveSpace();
            }
            sb.AppendLine("} ");
        }

        private void AddDetailsMethod()
        {
            sb.AppendLine();
            sb.AppendLine("//");
            sb.AppendLine("// GET: /" + EntityPropertyName + "/Details/5");
            sb.AppendLine();
            sb.AppendLine("public ViewResult Details(" + PropertyClassPrimaryKeyType + " ID)");
            sb.AppendLine("{");
            if (true)
            {
                sb.AddSpace();

                if (UsingDropDownLists)
                {
                    sb.AppendLine("var " + EntityPropertyName + " = db." + EntityPropertyName + "");
                    if (true)
                    {
                        sb.AddSpace();
                        foreach (GridChoice item in GridChoices)
                        {
                            sb.AppendLine(".Include(o => o." + item.PrimaryTable + ")");
                        }

                        sb.AppendLine(".Where(o => o." + SelectedEntitySet.EntityTypeLink.Keys[0] + " == ID);");
                        sb.RemoveSpace();
                    }
                    sb.AppendLine();
                    sb.AppendLine("" + PropertyClassName + "Detail " + PropertyClassName + "Detail = Get" + PropertyClassName + "DetailFrom" + EntityPropertyName + "(" + EntityPropertyName + ").Single();");
                    sb.AppendLine();
                    sb.AppendLine("return View(" + PropertyClassName + "Detail);");
                }
                else
                {
                    sb.AppendLine("" + PropertyClassName + " " + PropertyClassNameVariable + " = db." + EntityPropertyName + ".Single(o => o." + PropertyClassPrimaryKeyName + " == ID);");
                    sb.AppendLine("return View(" + PropertyClassNameVariable + ");");
                }
                sb.RemoveSpace();
            }
            sb.AppendLine("}");
        }

        private void AddDisposeMethod()
        {
            sb.AppendLine();
            sb.AppendLine("protected override void Dispose(bool disposing)");
            sb.AppendLine("{");
            if (true)
            {
                sb.AddSpace();
                sb.AppendLine("if(disposing) {");
                if (true)
                {
                    sb.AddSpace();
                    sb.AppendLine("db.Dispose();");
                    sb.RemoveSpace();
                }
                sb.AppendLine("}");
                sb.AppendLine("base.Dispose(disposing);");
                sb.RemoveSpace();
            }
            sb.AppendLine("}");
        }

        private void AddGetBooleanDropDownListOptionsMethod()
        {
            foreach (Searchable item in SearchChoices)
            {
                if (item.Property.Type.Contains("Boolean"))
                {
                    sb.AppendLine();
                    sb.AppendLine("private IEnumerable<SelectListItem> GetBooleanDropDownListOptions()");
                    sb.AppendLine("{");
                    if (true)
                    {
                        sb.AddSpace();
                        sb.AppendLine("List<SelectListItem> List = new List<SelectListItem>();");
                        sb.AppendLine(@"List.Add( new SelectListItem { Text = """", Value = """", Selected = true } );");
                        sb.AppendLine(@"List.Add( new SelectListItem { Text = ""Yes"", Value = ""Yes"", Selected = false } );");
                        sb.AppendLine(@"List.Add( new SelectListItem { Text = ""No"", Value = ""No"", Selected = false } );");
                        sb.AppendLine();
                        sb.AppendLine("return List;");
                        sb.RemoveSpace();
                    }
                    sb.AppendLine("}");
                    break;
                }
            }
        }

        private void AddGetNumericalDropDownListOptionsMethod()
        {
            foreach (Searchable item in SearchChoices)
            {
                if (Basics.IsNumericType(item.Property.Type))
                {
                    sb.AppendLine();
                    sb.AppendLine("private IEnumerable<SelectListItem> GetNumericalDropDownListOptions()");
                    sb.AppendLine("{");
                    if (true)
                    {
                        sb.AddSpace();
                        sb.AppendLine("List<SelectListItem> List = new List<SelectListItem>();");
                        sb.AppendLine(@"List.Add( new SelectListItem { Text = """", Value = """", Selected = true } );");
                        sb.AppendLine(@"List.Add( new SelectListItem { Text = ""Equals"", Value = ""="", Selected = false } );");
                        sb.AppendLine(@"List.Add( new SelectListItem { Text = ""Not Equal"", Value = ""!="", Selected = false } );");
                        sb.AppendLine(@"List.Add( new SelectListItem { Text = ""Greater Than"", Value = "">"", Selected = false } );");
                        sb.AppendLine(@"List.Add( new SelectListItem { Text = ""Greater Than Or Equal To"", Value = "">="", Selected = false } );");
                        sb.AppendLine(@"List.Add( new SelectListItem { Text = ""Less Than"", Value = ""<"", Selected = false } );");
                        sb.AppendLine(@"List.Add( new SelectListItem { Text = ""Less Than or Equal To"", Value = ""<="", Selected = false } );");
                        sb.AppendLine(@"List.Add( new SelectListItem { Text = ""Between Inclusive"", Value = ""Between Inclusive"", Selected = false } );");
                        sb.AppendLine(@"List.Add( new SelectListItem { Text = ""Between Exclusive"", Value = ""Between Exclusive"", Selected = false } );");
                        sb.AppendLine(@"List.Add( new SelectListItem { Text = ""Not Between Inclusive"", Value = ""Not Between Inclusive"", Selected = false } );");
                        sb.AppendLine(@"List.Add( new SelectListItem { Text = ""Not Between Exclusive"", Value = ""Not Between Exclusive"", Selected = false } );");
                        sb.AppendLine();
                        sb.AppendLine("return List;");
                        sb.RemoveSpace();
                    }
                    sb.AppendLine("}");
                    break;
                }
            }
        }

        private void AddGetDateDropDownListOptionsMethod()
        {
            // If there is a DateTime field add a loader for the date/time combo box.
            foreach (Searchable item in SearchChoices)
            {
                if (item.Property.Type.Contains("DateTime"))
                {
                    sb.AppendLine();
                    sb.AppendLine("private IEnumerable<SelectListItem> GetDateDropDownListOptions()");
                    sb.AppendLine("{");
                    if (true)
                    {
                        sb.AddSpace();
                        sb.AppendLine("List<SelectListItem> List = new List<SelectListItem>();");
                        sb.AppendLine(@"List.Add( new SelectListItem { Text = """", Value = """", Selected = true } );");
                        sb.AppendLine(@"List.Add( new SelectListItem { Text = ""Equals"", Value = ""Equals"", Selected = false } );");
                        sb.AppendLine(@"List.Add( new SelectListItem { Text = ""Before"", Value = ""Before"", Selected = false } );");
                        sb.AppendLine(@"List.Add( new SelectListItem { Text = ""After"", Value = ""After"", Selected = false } );");
                        sb.AppendLine(@"List.Add( new SelectListItem { Text = ""Between"", Value = ""Between"", Selected = false } );");
                        sb.AppendLine("");
                        sb.AppendLine("return List;");
                        sb.RemoveSpace();
                    }
                    sb.AppendLine("}");
                    break;
                }
            }
        }

        private void AddGetClassDetailFromEntityMethod()
        {
            sb.AppendLine();
            sb.AppendLine("private IQueryable<" + PropertyClassName + "Detail> Get" + PropertyClassName + "DetailFrom" + EntityPropertyName + "(IQueryable<" + PropertyClassName + "> " + EntityPropertyName + ")");
            sb.AppendLine("{");
            if (true)
            {
                sb.AddSpace();
                sb.AppendLine("return " + EntityPropertyName + ".Select(x => new " + PropertyClassName + "Detail");
                sb.AppendLine("{");
                if (true)
                {
                    sb.AddSpace();

                    foreach (GridChoice item in GridChoices)
                    {
                        // todo
                        //              ReportGroupGroupName = (x.ReportGroup == null ? "" : x.ReportGroup.GroupName),
                        //              ConnectionConnectionID = (x.Connection == null ? new Nullable<int>() : x.Connection.ConnectionID),
                        sb.AppendLine(item.PrimaryTable + item.DisplayColumnName + " = x." + item.PrimaryTable + "." + item.DisplayColumnName + ",");
                    }

                    foreach (EDMXParserV1.Property Property in SelectedEntitySet.EntityTypeLink.Properties)
                    {
                        bool GridChoiceFound = false;
                        foreach (GridChoice GridChoice in GridChoices)
                        {
                            if (GridChoice.ColumnName == Property.Name)
                            {
                                GridChoiceFound = true;
                                break;
                            }
                        }

                        if (GridChoiceFound == false)
                        {
                            sb.AppendLine(Property.Name + " = x." + Property.Name + ",");
                        }
                    }
                    sb.RemoveSpace();
                }
                sb.AppendLine("});");
                sb.RemoveSpace();
            }
            sb.AppendLine("}");
        }

        private void AddGetLastPageMethod(String PropertyClassName)
        {
            sb.AppendLine();
            sb.AppendLine("private int GetLastPage(" + PropertyClassName + "Index " + PropertyClassName + "Index)");
            sb.AppendLine("{");
            if (true)
            {
                sb.AddSpace();
                sb.AppendLine("int NumberOfPagesToSkip = " + PropertyClassName + "Index.RecordCount / " + PropertyClassName + "Index.RecordsPerPage;");
                sb.AppendLine("");
                sb.AppendLine("int RemainingRecords = " + PropertyClassName + "Index.RecordCount % " + PropertyClassName + "Index.RecordsPerPage;");
                sb.AppendLine("if (RemainingRecords == 0)");
                sb.AppendLine("{");
                if (true)
                {
                    sb.AddSpace();
                    sb.AppendLine("// perfect match.");
                    sb.AppendLine("// do nothing");
                    sb.RemoveSpace();
                }
                sb.AppendLine("}");
                sb.AppendLine("else");
                sb.AppendLine("{");
                if (true)
                {
                    sb.AddSpace();
                    sb.AppendLine("// there is a remainder so we need to show one more page.");
                    sb.AppendLine("NumberOfPagesToSkip = NumberOfPagesToSkip + 1;");
                    sb.RemoveSpace();
                }
                sb.AppendLine("}");
                sb.AppendLine();
                sb.AppendLine("return NumberOfPagesToSkip;");
                sb.RemoveSpace();
            }
            sb.AppendLine("}");
        }

        private void AddGetNumberOfPagesToSkipMethod(String PropertyClassName)
        {
            sb.AppendLine();
            sb.AppendLine("private static int GetNumberOfPagesToSkip(" + PropertyClassName + "Index " + PropertyClassName + "Index, int NumberOfPagesToSkip)");
            sb.AppendLine("{");
            if (true)
            {
                sb.AddSpace();
                sb.AppendLine("if (" + PropertyClassName + "Index.CurrentPage == 1)");
                sb.AppendLine("{");
                if (true)
                {
                    sb.AddSpace();
                    sb.AppendLine("NumberOfPagesToSkip = 0;");
                    sb.RemoveSpace();
                }
                sb.AppendLine("}");
                sb.AppendLine("else");
                sb.AppendLine("{");
                if (true)
                {
                    sb.AddSpace();
                    sb.AppendLine("NumberOfPagesToSkip = (" + PropertyClassName + "Index.CurrentPage - 1);");
                    sb.RemoveSpace();
                }
                sb.AppendLine("}");
                sb.AppendLine("return NumberOfPagesToSkip;");
                sb.RemoveSpace();
            }
            sb.AppendLine("}");
        }

        private void AddConfigurePagingMethod(String EntityPropertyName, String PropertyClassName)
        {
            sb.AppendLine();
            sb.AppendLine("private IQueryable<" + PropertyClassName + "> ConfigurePaging(" + PropertyClassName + "Index " + PropertyClassName + "Index, IQueryable<" + PropertyClassName + "> " + EntityPropertyName + ")");
            sb.AppendLine("{");
            if (true)
            {
                sb.AddSpace();
                sb.AppendLine("");
                sb.AppendLine("// Paging Section");
                sb.AppendLine("int NumberOfPagesToSkip = 0;");
                sb.AppendLine("");
                sb.AppendLine("int NumberOfRecordsToSkip = NumberOfPagesToSkip * " + PropertyClassName + "Index.RecordsPerPage;");
                sb.AppendLine("");
                sb.AppendLine(@"ModelState.Remove(""RecordCount"");");
                sb.AppendLine();
                sb.AppendLine(@"ModelState.Remove(""CurrentPage"");");
                sb.AppendLine();
                sb.AppendLine(PropertyClassName + "Index.RecordCount = " + EntityPropertyName + ".Count();");
                sb.AppendLine();
                sb.AppendLine("switch (" + PropertyClassName + "Index.PagingAction)");
                sb.AppendLine("{");
                if (true)
                {
                    sb.AddSpace();
                    sb.AppendLine(@"case ""First"":");
                    if (true)
                    {
                        sb.AddSpace();
                        sb.AppendLine("NumberOfPagesToSkip = 0;");
                        sb.AppendLine(PropertyClassName + "Index.CurrentPage = 1;");
                        sb.AppendLine();
                        sb.AppendLine("break;");
                        sb.RemoveSpace();
                    }
                    sb.AppendLine(@"case ""Last"":");
                    if (true)
                    {
                        sb.AddSpace();
                        sb.AppendLine(PropertyClassName + "Index.CurrentPage = GetLastPage(" + PropertyClassName + "Index);");
                        sb.AppendLine();
                        sb.AppendLine("NumberOfPagesToSkip = GetNumberOfPagesToSkip(" + PropertyClassName + "Index, NumberOfPagesToSkip);");
                        sb.AppendLine("");
                        sb.AppendLine("break;");
                        sb.RemoveSpace();
                    }
                    sb.AppendLine(@"case ""Next"":");
                    if (true)
                    {
                        sb.AddSpace();
                        sb.AppendLine("// filter options may have changed so need to make sure Next is a viable option.");
                        sb.AppendLine("int PotentialNextPage = " + PropertyClassName + "Index.CurrentPage + 1;");
                        sb.AppendLine("int PotentialNumberOfRecordsToSkip = PotentialNextPage * " + PropertyClassName + "Index.RecordsPerPage;");
                        sb.AppendLine();
                        sb.AppendLine("if (PotentialNumberOfRecordsToSkip >  " + PropertyClassName + "Index.RecordCount)");
                        sb.AppendLine("{");
                        if (true)
                        {
                            sb.AddSpace();
                            sb.AppendLine("// set it to the last page.");
                            sb.AppendLine(PropertyClassName + "Index.CurrentPage = GetLastPage(" + PropertyClassName + "Index);");
                            sb.RemoveSpace();
                        }
                        sb.AppendLine("}");
                        sb.AppendLine("else");
                        sb.AppendLine("{");
                        if (true)
                        {
                            sb.AddSpace();
                            sb.AppendLine(PropertyClassName + "Index.CurrentPage = PotentialNextPage;");
                            sb.RemoveSpace();
                        }
                        sb.AppendLine("}");
                        sb.AppendLine("NumberOfPagesToSkip = GetNumberOfPagesToSkip(" + PropertyClassName + "Index, NumberOfPagesToSkip);");
                        sb.AppendLine("break;");
                        sb.RemoveSpace();
                    }
                    sb.AppendLine(@"case ""Previous"":");
                    if (true)
                    {
                        sb.AddSpace();
                        sb.AppendLine("// filter options may have changed so need to make sure Next is a viable option.");
                        sb.AppendLine("int PreviousPotentialNextPage = " + PropertyClassName + "Index.CurrentPage - 1;");
                        sb.AppendLine("if (PreviousPotentialNextPage < 1)");
                        sb.AppendLine("{");
                        if (true)
                        {
                            sb.AddSpace();
                            sb.AppendLine(PropertyClassName + "Index.CurrentPage = 1;");
                            sb.RemoveSpace();
                        }
                        sb.AppendLine("}");
                        sb.AppendLine("else");
                        sb.AppendLine("{");
                        if (true)
                        {
                            sb.AddSpace();
                            sb.AppendLine("int PreviousPotentialNumberOfRecordsToSkip = PreviousPotentialNextPage * " + PropertyClassName + "Index.RecordsPerPage;");
                            sb.AppendLine("if (PreviousPotentialNumberOfRecordsToSkip > " + PropertyClassName + "Index.RecordCount)");
                            sb.AppendLine("{");
                            if (true)
                            {
                                sb.AddSpace();
                                sb.AppendLine("// set it to the last page.");
                                sb.AppendLine(PropertyClassName + "Index.CurrentPage = GetLastPage(" + PropertyClassName + "Index);");
                                sb.RemoveSpace();
                            }
                            sb.AppendLine("}");
                            sb.AppendLine("else");
                            sb.AppendLine("{");
                            if (true)
                            {
                                sb.AddSpace();
                                sb.AppendLine(PropertyClassName + "Index.CurrentPage = PreviousPotentialNextPage;");
                                sb.RemoveSpace();
                            }
                            sb.AppendLine("}");
                            sb.RemoveSpace();
                        }
                        sb.AppendLine("}");
                        sb.AppendLine("NumberOfPagesToSkip = GetNumberOfPagesToSkip(" + PropertyClassName + "Index, NumberOfPagesToSkip);");
                        sb.AppendLine("break;");
                        sb.RemoveSpace();
                    }
                    sb.AppendLine("default:");
                    if (true)
                    {
                        sb.AddSpace();
                        sb.AppendLine("break;");
                        sb.RemoveSpace();
                    }
                    sb.RemoveSpace();
                }
                sb.AppendLine("}");
                sb.AppendLine("");
                sb.AppendLine("NumberOfRecordsToSkip = NumberOfPagesToSkip * " + PropertyClassName + "Index.RecordsPerPage;");
                sb.AppendLine("");
                sb.AppendLine(EntityPropertyName + " = " + EntityPropertyName + ".Skip(NumberOfRecordsToSkip)");
                if (true)
                {
                    sb.AddSpace();
                    sb.AppendLine(".Take(" + PropertyClassName + "Index.RecordsPerPage);");
                    sb.RemoveSpace();
                }
                sb.AppendLine("return " + EntityPropertyName + ";");
                sb.RemoveSpace();
            }
            sb.AppendLine("}");
        }

        private static void AddSortCode(List<GridChoice> GridChoices, String EntityPropertyName, String PropertyClassName, SpaceStackStringBuilder sb, Sortable Sortable)
        {
            sb.AppendLine(@"if (" + PropertyClassName + @"Index.SortDirection == ""desc"")");
            sb.AppendLine("{");


            GridChoice CurrentGridChoice = null;
            foreach (GridChoice GridChoice in GridChoices)
            {
                if (GridChoice.ColumnName == Sortable.ColumnName)
                {
                    CurrentGridChoice = GridChoice;
                }
            }

            if (CurrentGridChoice.IsNull())
            {
                if (true)
                {
                    sb.AddSpace();
                    sb.AppendLine(EntityPropertyName + " = " + EntityPropertyName + ".OrderByDescending(x => x." + Sortable.ColumnName + ");");
                    sb.RemoveSpace();
                }
                sb.AppendLine("}");
                sb.AppendLine("else");
                sb.AppendLine("{");
                if (true)
                {
                    sb.AddSpace();
                    sb.AppendLine(EntityPropertyName + " = " + EntityPropertyName + ".OrderBy(x => x." + Sortable.ColumnName + ");");
                    sb.RemoveSpace();
                }
                sb.AppendLine("}");
            }
            else
            {
                if (true)
                {
                    sb.AddSpace();
                    sb.AppendLine(EntityPropertyName + " = " + EntityPropertyName + ".OrderByDescending(x => x." + CurrentGridChoice.PrimaryTable + "." + CurrentGridChoice.DisplayColumnName + ");");
                    sb.RemoveSpace();
                }
                sb.AppendLine("}");
                sb.AppendLine("else");
                sb.AppendLine("{");
                if (true)
                {
                    sb.AddSpace();
                    sb.AppendLine("" + EntityPropertyName + " = " + EntityPropertyName + ".OrderBy(x => x." + CurrentGridChoice.PrimaryTable + "." + CurrentGridChoice.DisplayColumnName + ");");
                    sb.RemoveSpace();
                }
                sb.AppendLine("}");
            }
        }

        private String GetConverterFromTypeName(String TypeName)
        {
            switch (TypeName)
            {
                case "Int32":
                    return "ToInt32";
                default:
                    Debug.WriteLine("GetConverterFromTypeName failure: " + TypeName);
                    break;
            }
            return "";
        }

        private void GetIndexBaseQuery()
        {
            sb.AppendLine("IQueryable<" + PropertyClassName + "> " + EntityPropertyName + " = db");
            sb.AddSpace();
            sb.AppendStart();
            sb.Append("." + EntityPropertyName + "");
            sb.RemoveSpace();

            foreach (GridChoice item in GridChoices)
            {
                sb.AppendLine();
                sb.AddSpace();
                sb.AppendStart();
                sb.Append(".Include(x => x." + item.PrimaryTable + ")");
                sb.RemoveSpace();
            }
            sb.Append(";");
            sb.AppendLine();
        }

        private String GetSelectListFromGridChoice(EDMXParserV1.Edmx Edmx, GridChoice GridChoice, String Default, EntitySet SelectedEntitySet)
        {
            StringBuilder sb = new StringBuilder();

            //foreach (EntitySet EntitySet in Edmx.Runtime.ConceptualModels.Schema.EntityContainer.EntitySets)
            //{
            //    if (EntitySet.EntityTypeLink.Name == GridChoice.PrimaryTable)
            //    {
            //        sb.Append("SelectList( db." + EntitySet.Name + @", """ + GridChoice.PrimaryKey + @""", """ + GridChoice.DisplayColumnName + @"""");
            //        if (Default.Length > 0)
            //        {
            //            sb.Append(" ," + Default + " ");
            //        }
            //        else
            //        {
            //            // do nothing
            //        }

            //        sb.Append(");");
            //        break;
            //    }
            //}
            foreach (NavigationProperty Item in SelectedEntitySet.EntityTypeLink.NavigationProperties)
            {
                if (GridChoice.PrimaryTable == Item.Name)
                {
                    foreach (EDMXParserV1.Property Property in Item.ToEntityType.Properties)
                    {
                        if (Property.Name == GridChoice.DisplayColumnName)
                        {
                            foreach (EntitySet EntitySet in Edmx.Runtime.ConceptualModels.Schema.EntityContainer.EntitySets)
                            {
                                if (EntitySet.EntityTypeLink.Name == Item.ToRole)
                                {
                                    sb.Append("SelectList( db." + EntitySet.Name + @", """ + GridChoice.PrimaryKey + @""", """ + GridChoice.DisplayColumnName + @"""");
                                    if (Default.Length > 0)
                                    {
                                        sb.Append(" ," + Default + " ");
                                    }
                                    else
                                    {
                                        // do nothing
                                    }

                                    sb.Append(");");
                                    break;
                                }
                            }
                        }
                    }
                }
            }
            return sb.ToString();
        }

        public override string ToString()
        {
            return sb.ToString();
        }
    }
}
